@@ -21,6 +21,7 @@
 
 #include <linux/module.h>
 #include <linux/kernel.h>
+#include <linux/version.h>
 #include <linux/init.h>
 #include <linux/fs.h>
 #include <linux/cdev.h>
@@ -30,7 +31,6 @@
 #include <linux/slab.h>
 #include <linux/io.h>
 #include <linux/grlib/maplib.h>
-#include <linux/grlib/devno.h>
 
 MODULE_LICENSE ("GPL");
 
@@ -47,10 +47,6 @@ MODULE_LICENSE ("GPL");
 
 #define MAPLIB_MAX 1
 
-#if (MAPLIB_MAX > MAPLIB_DEVCNT)
-#error MAPLIB: not enough CHAR device nodes
-#endif
-
 #define BLK_SIZE   0x20000
 #define BLK_SHIFT  17
 
@@ -94,6 +90,8 @@ struct maplib_priv {
 
 struct maplib_priv *maplib_privs = NULL;
 int maplib_count = 0;
+static struct class *maplib_class = NULL;
+static int dev_major = 0;
 
 static int maplib_open(struct inode *inode, struct file *file);
 static int maplib_release(struct inode *inode, struct file *file);
@@ -117,7 +115,7 @@ struct vm_operations_struct maplib_vops = {
  * mapped. This means that we must signal to all drivers to
  * stop using the memory.
  */
-void maplib_drv_unreg_all(struct maplib_priv *priv)
+static void maplib_drv_unreg_all(struct maplib_priv *priv)
 {
 	struct maplib_drv *drv = priv->drvs;
 	while ( drv ) {
@@ -368,7 +366,11 @@ static int maplib_mmap(struct file *filp, struct vm_area_struct *vma)
 	printk(KERN_DEBUG "        VM_flags:         0x%x\n", (unsigned int)vma->vm_flags);
 	printk(KERN_DEBUG "        VM_ops:           0x%x\n", (unsigned int)vma->vm_ops);
 #endif
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6,3,0)
 	vma->vm_flags |= VM_DONTEXPAND | VM_DONTDUMP;
+#else
+	vm_flags_set(vma, vma->vm_flags | VM_DONTEXPAND | VM_DONTDUMP);
+#endif
 	vma->vm_private_data = priv;
 	vma->vm_ops = &maplib_vops;
 
@@ -472,7 +474,7 @@ static int maplib_buf_setup(struct maplib_priv *priv, struct maplib_setup *setup
 
 	/*** Setup Buffers ***/
 
-	/* Calculate number of MMAP-Blocks required to fit all 
+	/* Calculate number of MMAP-Blocks required to fit all
 	 * user's blocks. A MMAP block is limited to a specific
 	 * size limited by the arch port. 128Kb seems standard.
 	 */
@@ -594,13 +596,19 @@ static int __init maplib_init (void)
 	maplib_count = MAPLIB_MAX;
 
 	/* Get Major Number of CHAR driver */
-	dev_id = MKDEV(MAPLIB_MAJOR, MAPLIB_MINOR);
-	result = register_chrdev_region(dev_id, maplib_count, "maplib");
+	result = alloc_chrdev_region(&dev_id, 0, maplib_count, "maplib");
 	if ( result < 0 ) {
 		printk(KERN_WARNING "MAPLIB: Failed to register CHAR region\n");
 		return 0;
 	}
 
+	dev_major = MAJOR(dev_id);
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6,4,0)
+	maplib_class = class_create(THIS_MODULE, "maplib");
+#else
+	maplib_class = class_create("maplib");
+#endif
 	/* Allocate private memory for all MAPLIB cores */
 	maplib_privs = (struct maplib_priv *)
 		kmalloc(sizeof(struct maplib_priv) * maplib_count, GFP_KERNEL);
@@ -614,12 +622,14 @@ static int __init maplib_init (void)
 		priv->devname[7] = '0' + i;
 
 		/* Init and Register CHAR driver */
-		dev_id = MKDEV(MAPLIB_MAJOR, MAPLIB_MINOR+i);
+		dev_id = MKDEV(dev_major, i);
 		cdev_init(&priv->cdev, &maplib_fops);
 		result = cdev_add(&priv->cdev, dev_id, 1);
 		if ( result ) {
 			printk(KERN_NOTICE "MAPLIB: Failed adding CHAR dev\n");
+			continue;
 		}
+		device_create(maplib_class, NULL, dev_id, NULL, "maplib%d", i);
 	}
 
 	printk(KERN_DEBUG "MAPLIB: max %d mmap-pools\n", maplib_count);
@@ -630,7 +640,6 @@ static int __init maplib_init (void)
 static void __exit maplib_exit (void)
 {
 	int i;
-	dev_t devno = MKDEV (MAPLIB_MAJOR, MAPLIB_MINOR);
 	struct maplib_priv *priv;
 
 	for (i = 0; i<maplib_count; i++) {
@@ -638,7 +647,14 @@ static void __exit maplib_exit (void)
 		cdev_del(&priv->cdev);
 	}
 
-	unregister_chrdev_region(devno, maplib_count);
+	for (i = 0; i < maplib_count; i++) {
+		device_destroy(maplib_class, MKDEV(dev_major, i));
+	}
+
+	class_unregister(maplib_class);
+	class_destroy(maplib_class);
+
+	unregister_chrdev_region(MKDEV(dev_major, 0), maplib_count);
 
 	if ( maplib_privs )
 		kfree(maplib_privs);
