@@ -21,6 +21,7 @@
 
 #include <linux/module.h>
 #include <linux/kernel.h>
+#include <linux/version.h>
 #include <linux/init.h>
 #include <linux/fs.h>
 #include <linux/cdev.h>
@@ -37,6 +38,7 @@
 #include <linux/of_irq.h>
 #include <linux/of_platform.h>
 #include <linux/dma-mapping.h>
+#include <linux/platform_device.h>
 #include <linux/clk.h>
 
 #ifdef CONFIG_SPARC_LEON
@@ -2633,10 +2635,9 @@ static int grspw_of_probe(struct platform_device *of_dev)
 	struct device_node *onp = of_dev->dev.of_node;
 	struct grspw_priv *priv;
 	struct clk *clk;
-	int i, size, err, len;
+	int i, size, err, rc;
 	unsigned int ctrl;
-	const int *pampopts, *pversion, *pindex, *pdevice;
-	int devid = 0;
+	u32 ampopts, version, device;
 	char *devname;
 
 	if ( grspw_count >= GRSPW_MAX )
@@ -2651,8 +2652,8 @@ static int grspw_of_probe(struct platform_device *of_dev)
 	/* Skip this device if user has set ampopts to zero, the device may
 	 * be used by another OS instance in AMP systems.
 	 */
-	pampopts = of_get_property(of_dev->dev.of_node, "ampopts", &len);
-	if ( pampopts && (len == 4) && (*pampopts == 0) )
+	rc = of_property_read_u32(of_dev->dev.of_node, "ampopts", &ampopts);
+	if ( !rc && (ampopts == 0) )
 		return 0;
 
 	priv = (struct grspw_priv *)kmalloc(sizeof(*priv), GFP_KERNEL);
@@ -2703,19 +2704,16 @@ static int grspw_of_probe(struct platform_device *of_dev)
 	priv->hwsup.ndma_chans = 1 + ((ctrl & GRSPW_CTRL_NCH) >> GRSPW_CTRL_NCH_BIT);
 
 	/* Find Device ID and Version */
-	priv->hwsup.hw_version = 0;
-	pversion = of_get_property(of_dev->dev.of_node, "version", &len);
-	if ( pversion && (len == 4) )
-		priv->hwsup.hw_version = *pversion;
-	pdevice = 0;
-	pdevice = of_get_property(of_dev->dev.of_node, "device", &len);
-	if ( pdevice && (len == 4) )
-		devid = *pdevice;
-	priv->hwsup.hw_version |= devid << 16;
-	if ( (devid == GAISLER_SPW2) || (devid == GAISLER_SPW2_DMA) ) {
+	version = 0;
+	device = 0;
+	of_property_read_u32(of_dev->dev.of_node, "version", &version);
+	of_property_read_u32(of_dev->dev.of_node, "device", &device);
+	priv->hwsup.hw_version = (device << 16) | version;
+
+	if ( (device == GAISLER_SPW2) || (device == GAISLER_SPW2_DMA) ) {
 		priv->hwsup.strip_adr = 1; /* All GRSPW2 can strip Address */
 		priv->hwsup.strip_pid = 1; /* All GRSPW2 can strip PID */
-		if ( devid == GAISLER_SPW2 )
+		if ( device == GAISLER_SPW2 )
 			devname = "GRSPW2";
 		else
 			devname = "GRSPW2_ROUTER";
@@ -2728,9 +2726,7 @@ static int grspw_of_probe(struct platform_device *of_dev)
 
 	/* Get device Index just for Information */
 	priv->core_index = 0;
-	pindex = of_get_property(of_dev->dev.of_node, "index", &len);
-	if ( pindex && (len == 4) )
-		priv->core_index = *pindex;
+	of_property_read_u32(of_dev->dev.of_node, "index", &priv->core_index);
 
 	printk(KERN_NOTICE "GRSPW[%d]: type=%s, core_index=%d, vadr_regs=%p\n",
 		priv->index, devname, priv->core_index, priv->regs);
@@ -2785,8 +2781,11 @@ static int grspw_of_probe(struct platform_device *of_dev)
 error:
 	return err;
 }
-
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6,11,0)
 static int grspw_of_remove(struct platform_device *of_dev)
+#else
+static void grspw_of_remove(struct platform_device *of_dev)
+#endif
 {
 	struct grspw_priv *priv = dev_get_drvdata(&of_dev->dev);
 
@@ -2801,7 +2800,9 @@ static int grspw_of_remove(struct platform_device *of_dev)
 	kfree(priv);
 	dev_set_drvdata(&of_dev->dev, NULL);
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6,11,0)
 	return 0;
+#endif
 }
 
 static const struct of_device_id grspw_of_match[] = {
