@@ -29,16 +29,17 @@
 #include <linux/ioctl.h>
 #include <linux/slab.h>
 
+#include <linux/of.h>
 #include <linux/of_device.h>
 #include <linux/of_platform.h>
 #include <linux/version.h>
+#include <linux/platform_device.h>
 
 #include <linux/grlib/grspw_router.h>
-#include <linux/grlib/devno.h>
 
 #define REG_READ(adr) (*(volatile u32 *)(adr))
 #define REG_WRITE(adr, value) (*(volatile u32 *)(adr) = (value))
-
+#define SPWROUTER_DEVCNT       4
 struct router_regs {
 	u32 resv1;		/* 0x000 */
 	u32 psetup[255];	/* 0x004 */
@@ -67,6 +68,8 @@ struct router_priv {
 };
 
 static int router_count;
+static struct class *router_class = NULL;
+static int dev_major = 0;
 
 static int router_open(struct inode *inode, struct file *file);
 static int router_release(struct inode *inode, struct file *file);
@@ -111,7 +114,7 @@ static int router_release(struct inode *inode, struct file *file)
 	return 0;
 }
 
-void router_hwinfo(struct router_priv *priv, struct router_hw_info *hwinfo)
+static void router_hwinfo(struct router_priv *priv, struct router_hw_info *hwinfo)
 {
 	unsigned int tmp;
 
@@ -129,7 +132,7 @@ void router_hwinfo(struct router_priv *priv, struct router_hw_info *hwinfo)
 	hwinfo->iid         = (tmp >>  0) & 0xff;
 }
 
-int router_config_set(struct router_priv *priv, struct router_config *cfg)
+static int router_config_set(struct router_priv *priv, struct router_config *cfg)
 {
 	int i;
 
@@ -162,7 +165,7 @@ int router_config_set(struct router_priv *priv, struct router_config *cfg)
 	return 0;
 }
 
-int router_config_read(struct router_priv *priv, struct router_config *cfg)
+static int router_config_read(struct router_priv *priv, struct router_config *cfg)
 {
 	int i;
 
@@ -176,7 +179,7 @@ int router_config_read(struct router_priv *priv, struct router_config *cfg)
 	return 0;
 }
 
-int router_routes_set(struct router_priv *priv, struct router_routes *routes)
+static int router_routes_set(struct router_priv *priv, struct router_routes *routes)
 {
 	int i;
 	for (i=0; i<224; i++)
@@ -184,7 +187,7 @@ int router_routes_set(struct router_priv *priv, struct router_routes *routes)
 	return 0;
 }
 
-int router_routes_read(struct router_priv *priv, struct router_routes *routes)
+static int router_routes_read(struct router_priv *priv, struct router_routes *routes)
 {
 	int i;
 	for (i=0; i<224; i++)
@@ -192,31 +195,31 @@ int router_routes_read(struct router_priv *priv, struct router_routes *routes)
 	return 0;
 }
 
-int router_ps_set(struct router_priv *priv, struct router_ps *ps)
+static int router_ps_set(struct router_priv *priv, struct router_ps *ps)
 {
 	int i;
 	unsigned int *p = &ps->ps[0];
-	for (i=0; i<255; i++,p++) 
+	for (i=0; i<255; i++,p++)
 		REG_WRITE(&priv->regs->psetup[i], *p);
 	return 0;
 }
 
-int router_ps_read(struct router_priv *priv, struct router_ps *ps)
+static int router_ps_read(struct router_priv *priv, struct router_ps *ps)
 {
 	int i;
 	unsigned int *p = &ps->ps[0];
-	for (i=0; i<255; i++,p++) 
+	for (i=0; i<255; i++,p++)
 		REG_WRITE(&priv->regs->psetup[i], *p);
 	return 0;
 }
 
-int router_we_set(struct router_priv *priv, int we)
+static int router_we_set(struct router_priv *priv, int we)
 {
 	REG_WRITE(&priv->regs->cfgwe, we & 0x1);
 	return 0;
 }
 
-int router_port_ctrl(struct router_priv *priv, struct router_port *port)
+static int router_port_ctrl(struct router_priv *priv, struct router_port *port)
 {
 	unsigned int ctrl, sts;
 
@@ -244,19 +247,19 @@ int router_port_ctrl(struct router_priv *priv, struct router_port *port)
 	return 0;
 }
 
-int router_cfgsts_set(struct router_priv *priv, unsigned int cfgsts)
+static int router_cfgsts_set(struct router_priv *priv, unsigned int cfgsts)
 {
 	REG_WRITE(&priv->regs->cfgsts, cfgsts);
 	return 0;
 }
 
-int router_cfgsts_read(struct router_priv *priv, unsigned int *cfgsts)
+static int router_cfgsts_read(struct router_priv *priv, unsigned int *cfgsts)
 {
 	*cfgsts = REG_READ(&priv->regs->cfgsts);
 	return 0;
 }
 
-int router_tc_read(struct router_priv *priv, unsigned int *tc)
+static int router_tc_read(struct router_priv *priv, unsigned int *tc)
 {
 	*tc = REG_READ(&priv->regs->timecode);
 	return 0;
@@ -437,7 +440,7 @@ static int router_of_probe(struct platform_device *of_dev)
 
 	/* Register character device in registered region */
 	priv->minor = router_count;
-	dev_id = MKDEV(SPWROUTER_MAJOR, SPWROUTER_MINOR + priv->minor);
+	dev_id = MKDEV(dev_major, priv->minor);
 	cdev_init(&priv->cdev, &router_fops);
 	result = cdev_add(&priv->cdev, dev_id, 1);
 	if ( result ) {
@@ -455,6 +458,8 @@ static int router_of_probe(struct platform_device *of_dev)
 	priv->dev = &of_dev->dev;
 	dev_set_drvdata(&of_dev->dev, priv);
 
+	device_create(router_class, NULL, dev_id, NULL, "spwrouter%d", priv->minor);
+
 	router_count++;
 
 	printk (KERN_INFO "GRSPW_ROUTER[%d]: registered char device\n", priv->minor);
@@ -467,8 +472,11 @@ static int router_of_probe(struct platform_device *of_dev)
 error:
 	return err;
 }
-
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6,11,0)
 static int router_of_remove(struct platform_device *of_dev)
+#else
+static void router_of_remove(struct platform_device *of_dev)
+#endif
 {
 	struct router_priv *priv = dev_get_drvdata(&of_dev->dev);
 
@@ -477,7 +485,9 @@ static int router_of_remove(struct platform_device *of_dev)
 	kfree(priv);
 	dev_set_drvdata(&of_dev->dev, NULL);
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6,11,0)
 	return 0;
+#endif
 }
 
 static const struct of_device_id router_of_match[] = {
@@ -510,14 +520,20 @@ static int __init grspw_router_init(void)
 	int result;
 	dev_t dev_id;
 
-	/* Register CHAR driver region */
-	dev_id = MKDEV(SPWROUTER_MAJOR, SPWROUTER_MINOR);
-	result = register_chrdev_region(dev_id, SPWROUTER_DEVCNT, "grspw_router");
+	/* Get Major Number of CHAR driver */
+	result = alloc_chrdev_region(&dev_id, 0, SPWROUTER_DEVCNT, "grspw_router");
 	if ( result < 0 ) {
 		printk(KERN_WARNING "ROUTER: Failed to register CHAR region\n");
 		return 0;
 	}
 
+	dev_major = MAJOR(dev_id);
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6,4,0)
+	router_class = class_create(THIS_MODULE, "grspw_router");
+#else
+	router_class = class_create("grspw_router");
+#endif
 	router_count = 0;
 
 	return platform_driver_register(&router_of_driver);
@@ -525,11 +541,17 @@ static int __init grspw_router_init(void)
 
 static void __exit grspw_router_cleanup(void)
 {
-	dev_t dev_id;
+	int i;
+
+	for (i = 0; i < SPWROUTER_DEVCNT; i++) {
+		device_destroy(router_class, MKDEV(dev_major, i));
+	}
+
+	class_unregister(router_class);
+	class_destroy(router_class);
 
 	/* Unregister CHAR driver region */
-	dev_id = MKDEV(SPWROUTER_MAJOR, SPWROUTER_MINOR);
-	unregister_chrdev_region(dev_id, SPWROUTER_DEVCNT);
+	unregister_chrdev_region(MKDEV(dev_major, 0), SPWROUTER_DEVCNT);
 
 	platform_driver_unregister(&router_of_driver);
 }
