@@ -22,6 +22,7 @@
 
 #include <linux/module.h>
 #include <linux/kernel.h>
+#include <linux/version.h>
 #include <linux/init.h>
 #include <linux/fs.h>
 #include <linux/cdev.h>
@@ -39,11 +40,6 @@
 #include <linux/grlib/grspw.h>
 #include <linux/grlib/grspw_user.h>
 #include <linux/grlib/maplib.h>
-#include <linux/grlib/devno.h>
-
-#if (GRSPW_MAX > GRSPWU_DEVCNT)
-#error GRSPW: not enough CHAR device nodes
-#endif
 
 #ifndef init_MUTEX
 #define init_MUTEX(a) sema_init(a, 1);
@@ -106,6 +102,8 @@ struct grspw_bufcfg grspwu_def_bufcfg =
 };
 
 static struct grspwu_priv *privu_tab[GRSPW_MAX];
+static struct class *grspw_class = NULL;
+static int dev_major = 0;
 
 static int grspwu_open(struct inode *inode, struct file *file);
 static int grspwu_release(struct inode *inode, struct file *file);
@@ -212,7 +210,7 @@ static int grspwu_release(struct inode *inode, struct file *file)
 }
 
 /* Configure number of packet structures in driver (always called in stopped mode) */
-int grspwu_bufcfg(struct grspwu_priv *priv, struct grspw_bufcfg *bufcfg)
+static int grspwu_bufcfg(struct grspwu_priv *priv, struct grspw_bufcfg *bufcfg)
 {
 	/* Check buf configuration */
 	if ( (bufcfg->rx_pkts < 1) || (bufcfg->tx_pkts < 1) )
@@ -246,7 +244,7 @@ int grspwu_bufcfg(struct grspwu_priv *priv, struct grspw_bufcfg *bufcfg)
 }
 
 /* Configure Core and DMA Channels, RMAP and TimeCode setup */
-int grspwu_config_set(struct grspwu_priv *priv, struct grspw_config *cfg)
+static int grspwu_config_set(struct grspwu_priv *priv, struct grspw_config *cfg)
 {
 	int rmap_dstkey;
 	int i, retval;
@@ -321,7 +319,7 @@ int grspwu_config_set(struct grspwu_priv *priv, struct grspw_config *cfg)
 	return retval;
 }
 
-void grspwu_config_read(struct grspwu_priv *priv, struct grspw_config *cfg)
+static void grspwu_config_read(struct grspwu_priv *priv, struct grspw_config *cfg)
 {
 	int i;
 	int rmap_dstkey;
@@ -345,7 +343,7 @@ void grspwu_config_read(struct grspwu_priv *priv, struct grspw_config *cfg)
 	}
 }
 
-void grspwu_stats_read(struct grspwu_priv *priv, struct grspw_stats *stats)
+static void grspwu_stats_read(struct grspwu_priv *priv, struct grspw_stats *stats)
 {
 	int i;
 
@@ -359,7 +357,7 @@ void grspwu_stats_read(struct grspwu_priv *priv, struct grspw_stats *stats)
 	}
 }
 
-void grspwu_stats_clr(struct grspwu_priv *priv)
+static void grspwu_stats_clr(struct grspwu_priv *priv)
 {
 	int i;
 
@@ -371,7 +369,7 @@ void grspwu_stats_clr(struct grspwu_priv *priv)
 	}
 }
 
-int grspwu_link_ctrl(struct grspwu_priv *priv, struct grspw_link_ctrl *ctrl)
+static int grspwu_link_ctrl(struct grspwu_priv *priv, struct grspw_link_ctrl *ctrl)
 {
 	int clkdiv, stscfg;
 
@@ -386,8 +384,8 @@ int grspwu_link_ctrl(struct grspwu_priv *priv, struct grspw_link_ctrl *ctrl)
 	return 0;
 }
 
-int grspwu_port_ctrl(struct grspwu_priv *priv, int options)
-{	
+static int grspwu_port_ctrl(struct grspwu_priv *priv, int options)
+{
 	int port;
 
 
@@ -405,7 +403,7 @@ int grspwu_port_ctrl(struct grspwu_priv *priv, int options)
 	return 0;
 }
 
-void grspwu_link_state_get(struct grspwu_priv *priv, struct grspw_link_state *state)
+static void grspwu_link_state_get(struct grspwu_priv *priv, struct grspw_link_state *state)
 {
 	int clkdiv;
 
@@ -428,7 +426,7 @@ void grspwu_link_state_get(struct grspwu_priv *priv, struct grspw_link_state *st
 }
 
 /* Set TCTRL and TIMECNT, and Send Time-code */
-void grspwu_tc_send(struct grspwu_priv *priv, unsigned int options)
+static void grspwu_tc_send(struct grspwu_priv *priv, unsigned int options)
 {
 	int time;
 
@@ -443,13 +441,14 @@ void grspwu_tc_send(struct grspwu_priv *priv, unsigned int options)
 		grspw_tc_tx(priv->dh);
 }
 
-void grspwu_tc_read(struct grspwu_priv *priv, int *tc)
+static void grspwu_tc_read(struct grspwu_priv *priv, int *tc)
 {
 	*tc = -1;
 	grspw_tc_time(priv->dh, tc);
 }
 
-void grspwu_qpktcnt_read(struct grspwu_priv *priv, struct grspw_qpktcnt *qpktcnt)
+static void grspwu_qpktcnt_read(struct grspwu_priv *priv,
+				struct grspw_qpktcnt *qpktcnt)
 {
 	int i;
 	for (i=0; i<priv->chan_cnt; i++) {
@@ -470,7 +469,7 @@ void grspwu_qpktcnt_read(struct grspwu_priv *priv, struct grspw_qpktcnt *qpktcnt
 	}
 }
 
-int grspwu_start(struct grspwu_priv *priv, int options)
+static int grspwu_start(struct grspwu_priv *priv, int options)
 {
 	int size, i;
 	struct grspw_pkt *pktbase;
@@ -621,7 +620,7 @@ void grspwu_unmap(int idx, void *p)
 	grspwu_stop(priv);
 }
 
-int grspwu_rx_wait(struct grspwu_priv *priv, struct grspw_rx_wait_chan *wait)
+static int grspwu_rx_wait(struct grspwu_priv *priv, struct grspw_rx_wait_chan *wait)
 {
 	int rc;
 
@@ -645,7 +644,7 @@ int grspwu_rx_wait(struct grspwu_priv *priv, struct grspw_rx_wait_chan *wait)
 		return 0;
 }
 
-int grspwu_tx_wait(struct grspwu_priv *priv, struct grspw_tx_wait_chan *wait)
+static int grspwu_tx_wait(struct grspwu_priv *priv, struct grspw_tx_wait_chan *wait)
 {
 	int rc;
 
@@ -847,7 +846,7 @@ static long grspwu_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 }
 
 /* Reclaim Packets from one particular DMA Channel to user space */
-int grspwu_reclaim(
+static int grspwu_reclaim(
 	struct grspwu_priv *priv, int dmachan,
 	struct grspw_rtxpkt __user *dst, int max)
 {
@@ -944,7 +943,7 @@ out:
 }
 
 /* Read Received Packets from one particular DMA Channel to user space */
-int grspwu_recv(
+static int grspwu_recv(
 	struct grspwu_priv *priv, int dmachan,
 	struct grspw_rrxpkt __user *dst, int max)
 {
@@ -1040,7 +1039,7 @@ out:
 	return retval;
 }
 
-int grspwu_send(struct grspwu_priv *priv, int dmachan,
+static int grspwu_send(struct grspwu_priv *priv, int dmachan,
 		struct grspw_wtxpkt __user *src, int cnt)
 {
 	struct grspw_list txpkts;
@@ -1186,7 +1185,7 @@ out:
 	return retval;
 }
 
-int grspwu_prepare(struct grspwu_priv *priv, int dmachan,
+static int grspwu_prepare(struct grspwu_priv *priv, int dmachan,
 		struct grspw_wrxpkt __user *src, int cnt)
 {
 	struct grspw_list readypkts;
@@ -1453,7 +1452,7 @@ static ssize_t grspwu_write(struct file *filp, const char __user *buff, size_t c
 	}
 }
 
-void *grspwu_dev_add(int index)
+static void *grspwu_dev_add(int index)
 {
 	struct grspwu_priv *priv;
 	int err;
@@ -1470,7 +1469,7 @@ void *grspwu_dev_add(int index)
 	priv->name[6] = '0' + priv->index;
 
 	/* Init and Register CHAR driver */
-	dev_id = MKDEV(GRSPWU_MAJOR, GRSPWU_MINOR+priv->index);
+	dev_id = MKDEV(dev_major, priv->index);
 	cdev_init(&priv->cdev, &grspwu_fops);
 	err = cdev_add(&priv->cdev, dev_id, 1);
 	if ( err ) {
@@ -1480,10 +1479,12 @@ void *grspwu_dev_add(int index)
 		return NULL;
 	}
 
+	device_create(grspw_class, NULL, dev_id, NULL, "grspw%d", priv->index);
+
 	return priv;
 }
 
-void grspwu_dev_del(int index, void *data)
+static void grspwu_dev_del(int index, void *data)
 {
 	struct grspwu_priv *priv = data;
 
@@ -1501,15 +1502,21 @@ static int __init grspwu_init (void)
 	dev_t dev_id;
 
 	/* Get Major Number of CHAR driver */
-	dev_id = MKDEV(GRSPWU_MAJOR, GRSPWU_MINOR);
-	result = register_chrdev_region(dev_id, GRSPWU_DEVCNT, "grspw");
+	result = alloc_chrdev_region(&dev_id, 0, GRSPW_MAX, "grspw");
 	if ( result < 0 ) {
 		printk(KERN_WARNING "GRSPWU: Failed to register CHAR region\n");
 		return 0;
 	}
 
+	dev_major = MAJOR(dev_id);
+
 	memset(privu_tab, 0, sizeof(privu_tab));
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6,4,0)
+	grspw_class = class_create(THIS_MODULE, "grspw");
+#else
+	grspw_class = class_create("grspw");
+#endif
 	grspw_initialize_user(grspwu_dev_add,grspwu_dev_del);
 
 	return 0;
@@ -1518,7 +1525,6 @@ static int __init grspwu_init (void)
 static void __exit grspwu_exit (void)
 {
 	int i;
-	dev_t devno = MKDEV (GRSPWU_MAJOR, GRSPWU_MINOR);
 	struct grspwu_priv *priv;
 
 	/* Delete devices */
@@ -1529,7 +1535,14 @@ static void __exit grspwu_exit (void)
 	}
 	grspw_initialize_user(NULL, NULL);
 
-	unregister_chrdev_region(devno, GRSPWU_DEVCNT);
+	for (i = 0; i < GRSPW_MAX; i++) {
+		device_destroy(grspw_class, MKDEV(dev_major, i));
+	}
+
+	class_unregister(grspw_class);
+	class_destroy(grspw_class);
+
+	unregister_chrdev_region(MKDEV(dev_major, 0), GRSPW_MAX);
 }
 
 module_init(grspwu_init);
