@@ -65,7 +65,7 @@ int grspw_open(struct grspw_dev *dev, int idx)
 	sprintf(devname, "/dev/grspw%d", idx);
 	fd = open(devname, O_RDWR);
 	if ( fd < 0 ) {
-		PRINT_ERR("Failed to open GRSPW device %d: %d\n", idx, errno);
+		PRINT_ERR("Failed to open GRSPW device %d (%s): %s\n", idx, devname, strerror(errno));
 		return -1;
 	}
 	dev->fd = fd;
@@ -160,7 +160,7 @@ void grspw_cfg_print(struct grspw_hw_sup *hw, struct grspw_config *cfg)
 	printf(" DMA CHANNEL COUNT:       %d\n", hw->ndma_chans);
 	printf(" HARDWARE VERSION:        0x%08x\n", hw->hw_version);
 
-	printf("SOFWTARE CONFIGURATION:\n");
+	printf("SOFTWARE CONFIGURATION:\n");
 	printf(" Promiscuous Mode:        %s\n", cfg->adrcfg.promiscuous ? "YES" : "NO");
 	printf(" Default Node Address:    0x%02x\n", cfg->adrcfg.def_addr);
 	printf(" Default Node Mask:       0x%02x\n", cfg->adrcfg.def_mask);
