@@ -51,7 +51,7 @@ int mmaplib_init(int idx, int blk_size, int blk_cnt)
 	sprintf(devname, "/dev/maplib%d", idx);
 	map_fd = open(devname, O_RDWR);
 	if ( map_fd < 0 ) {
-		printf("Failed to open MMAPLib\n");
+		perror("Failed to open MMAPLib");
 		return -1;
 	}
 
