@@ -930,13 +930,37 @@ int grspw_process(struct grspw_device *device, int options, int stop)
 	return 0;
 }
 
+int timeval_subtract(struct timeval *result, struct timeval *x,
+                     struct timeval *y)
+{
+	/* Perform the carry for the later subtraction by updating y. */
+	if (x->tv_usec < y->tv_usec) {
+		int nsec = (y->tv_usec - x->tv_usec) / 1000000 + 1;
+		y->tv_usec -= 1000000 * nsec;
+		y->tv_sec += nsec;
+	}
+	if (x->tv_usec - y->tv_usec > 1000000) {
+		int nsec = (x->tv_usec - y->tv_usec) / 1000000;
+		y->tv_usec += 1000000 * nsec;
+		y->tv_sec -= nsec;
+	}
+
+	/* Compute the time remaining to wait.
+		tv_usec is certainly positive. */
+	result->tv_sec = x->tv_sec - y->tv_sec;
+	result->tv_usec = x->tv_usec - y->tv_usec;
+
+	/* Return 1 if result is negative. */
+	return x->tv_sec < y->tv_sec;
+}
+
 int main(int argc, char *argv[])
 {
 	int nlinks = ROUTER_SPWLINK_NO;
 	int usage_ok = 0;
 
 	int i, result, shutdown, stop, devno, options, tot;
-	struct timeval t0, t1, t2;
+	struct timeval t0, t1, t2, tr;
 	double time;
 	int hdrblk = 0;
 	int initiator, loopcnt, do_stop, cnt;
@@ -1185,8 +1209,8 @@ int main(int argc, char *argv[])
 	printf(" t0.tv_usec:    %ld\n", (long)t0.tv_usec);
 	printf(" t1.tv_sec:     %u\n", (unsigned int)t1.tv_sec);
 	printf(" t1.tv_usec:    %ld\n", (long)t1.tv_usec);
-	time = (float)t1.tv_sec + (float)t1.tv_usec/1000000;
-	time = time - (float)t0.tv_sec - (float)t0.tv_usec/1000000;
+	timeval_subtract(&tr, &t1, &t0);
+	time = (double)tr.tv_sec + (double)tr.tv_usec / 1000000.0;
 	printf(" Time:          %f seconds\n", time);
 	printf(" Packets sent:  %d packets\n", tot);
 	printf(" Packet size:   %d bytes in total\n", PKT_SIZE);
